; Sample documentation http://neuroph.sourceforge.net/tutorials/ChickenPricePredictionTutorial.htm by  Dr.V.Steinhauer
; Original java version http://neuroph.sourceforge.net/samples/ChickenPricePrediction.zip by  Dr.V.Steinhauer
; Clojure version by Bojan Jovicic (http://bojanjovicic.com) with help of Vitomir Kovanovic (http://blog.kovanovic.info/)

(ns org.neuroph.contrib.samples.chicken
  (:import [java.util Vector Date])
  (:import org.neuroph.core.NeuralNetwork)
  (:import org.neuroph.nnet.MultiLayerPerceptron)
  (:import org.neuroph.core.learning.TrainingSet)
  (:import org.neuroph.core.learning.TrainingElement)
  (:import org.neuroph.core.learning.SupervisedTrainingElement))

(defn get-data [] [77.77, 76.85, 77.25, 79.15, 81.23, 82.04, 83.46, 85.71, 88.25, 88.42, 88.40, 87.54, 87.02, 87.25, 86.7, 85.73, 85.38, 86.96, 88.17, 88.56, 86.77, 82.85, 82.13])
(defn get-data-max [data] (* 1.2 (reduce max data)))
(defn get-data-min [data] (* 0.8 (reduce min data)))

(defn prepare-data [data data-max data-min] (vec (map #(/ (- % data-min) data-max) data)))

(defn create-network []
  (let [mlp (MultiLayerPerceptron. (Vector. [4 9 1]))]
    (doto (.getLearningRule mlp)
      (.setMaxError 0.001)
      (.setLearningRate 0.7)
      (.setMaxIterations 100000))
    mlp))

(defn load-training-data [data]
  (let [training-set (TrainingSet.)]
    (doall
      (map
        #(let [params (subvec data % (+ % 5))
               ste (SupervisedTrainingElement. (Vector. (vec (take 4 params))) (Vector. [(last params)]))]
           (.addElement training-set ste))
        (range (- (count data) 5))))
    training-set))

(defn load-test-data [data]
  (let [test-set (TrainingSet.)
        start-element (- (count data) 5)
        params (subvec data start-element (+ start-element 4))
        te (TrainingElement. (Vector. (vec params)))] 
    (.addElement test-set te)
    test-set))

(defn get-network-value [data]
  (let [data-max (get-data-max data)
        data-min (get-data-min data)
        normalized-data (prepare-data data data-max data-min)
        training-set (load-training-data normalized-data)
        test-set (load-test-data normalized-data)
        network (doto (create-network)
                  (.learnInSameThread training-set)
                  (.setInput (.getInput (first (.trainingElements test-set))))
                  (.calculate))]
    (+ (* (first (.getOutput network)) data-max) data-min)))

(defn get-middle-value [data] (/ (reduce + (subvec data (- (count data) 5) (- (count data) 1))) 4))

(defn show-outputs [data]
  (let [middle-value (get-middle-value data)
        predicted-value (get-network-value data)]
    (if (< predicted-value middle-value) 
      (println "You can buy. It will not be expensive.")
      (println "Chicken might be more expensive!"))
    (println "Middle value:" middle-value)
    (println "Artificial neural network (multi layer perceptron value):" predicted-value)))

(defn -main []
  (show-outputs (get-data))) 